#!/usr/bin/perl

use DBI;
use Net::Ping;
use Time::HiRes;
use Getopt::Std;
use POSIX qw(strftime);
use Errno qw(ENOENT);
use GD;
use GD::Graph::points;
use Data::Dumper;
require '/etc/mrlg.cfg';

my %opts;

my $graphimage = "$webdir/latgraph.png";
my $gwidth = 1024;
my $gheight = 768;
my $ymax = 500;
my $title = 'Endpoint Latency Statistics';
my @manhosts;

sub db_connect($) {
  my $verbose = shift;
                                                                                                           
  return $global_dbh if (defined $global_dbh);
                                                                                                           
  my $dsn = "DBI:$database_type:database=$database_name:host=$database_host:port=$database_port";
  print "DBI Connecting to $dsn\n" if $verbose;
                                                                                                           
  # Note: We do all manual error checking for db errors
  my $dbh = DBI->connect($dsn, $database_user, $database_pass,
                         { PrintError => 0, RaiseError => $verbose });
                                                                                                           
  $global_dbh = $dbh;
  return $global_dbh;
}
                                                                                                           
sub db_disconnect {
  $global_dbh->disconnect() if (defined $global_dbh);
  $global_dbh = undef;
  return 0;
}

sub get_hosts() {

	if (@manhosts) {
		push @$hosts, @manhosts;
	}else{
		$hosts = $dbh->selectcol_arrayref("SELECT DISTINCT host from latency");
	}
}

sub average($) {
	my $high=0;
	my $low=1000;
	my $lowcount=0;
	my $highcount=0;
	# Acceptable latency level
	$lat=50 unless $lat;
	
	foreach my $latency (@_) {
	        if ($latency > $high) { $high = $latency }
	        if ($latency < $low) { $low = $latency }
	        if ($latency > $lat) {
	                $highcount++;
	        } else {
	                $lowcount++;
	        }
	        $count++;
	        $total += $latency;
	}
	
	$average = sprintf "%.2f", $total / $count;
	my $over = $highcount / $count;
	my $percentover = sprintf "%.2f", $over * 100;
	return ($high, $low, $highcount, $lowcount, $percentover, $average, $lat, $count);

}

sub make_graphs() {
	my $readdb;	
	if ($tframe > 172800) {
		$readdb = 'latreport';
	} else {
		$readdb = 'latency';
	}

	if ($tframe) {
		$query = ("SELECT DISTINCT time FROM $readdb WHERE time > ( NOW() - INTERVAL $tframe SECOND) ORDER BY time ASC")
	} else {
		$query = ("SELECT DISTINCT time FROM latency ORDER BY time ASC");
	}
	$time = $dbh->selectcol_arrayref($query) or goto DB_FAILURE;
	push @data, [ @$time ];
	my $thishost;
	foreach my $host (@$hosts) {  
		if ($tframe) {
			$query = ("SELECT latency FROM $readdb WHERE host = '$host' AND time > ( NOW() - INTERVAL $tframe SECOND) ORDER BY time ASC");
		} else {
			$query = ("SELECT latency FROM latency WHERE host = '$host' ORDER BY time ASC");
		}
	    $thishost = $dbh->selectcol_arrayref($query) or goto DB_FAILURE;
		if ($tframe > 172800) {
			$query2 = ("SELECT latency FROM latency WHERE host = '$host'");
	    		$thishost2 = $dbh->selectcol_arrayref($query2) or goto DB_FAILURE;
			@$thishost = (@$thishost, @$thishost2);
		}
	    push @data, [ @$thishost ];
	} 

	my ($high, $low, $highcount, $lowcount, $percentover, $average, $lat, $count) = &average(@$thishost);
	my ($avxpos, $avypos) = (60,35);

	my $graph = GD::Graph::points->new($gwidth, $gheight);
	$graph->set(
		x_label			=> 'Time',
		y_label 		=> 'Milliseconds',
		y_max_value 		=> $ymax,
		y_min_value 		=> 0,
		title   		=> $title,
		#dclrs   		=> ['blue', 'green', 'red' ],
		markers			=> [ 3, 4, 5, 6, 7, 8, 9, 10, 1 ],
		#x_tick_number		=> 8,
		x_labels_vertical	=> 1,
		y_tick_number		=> 8,
		lg_cols			=> 2,
		transparent		=> 0,
		x_label_skip 		=> ($#$time / 30),
	) or warn $graph->error;

	$graph->set_legend_font(GD::gdMediumBoldFont);
	$graph->set_legend(@$hosts);
	my $image = $graph->plot(\@data) or die $graph->error;

	if (@$hosts == 1) {
		my $ltime = localtime();
		$overlay = newFromPngData GD::Image($image->png);
		$black=$overlay->colorAllocate(0,0,0);
		$white=$overlay->colorAllocate(255,255,255);
		$overlay->filledRectangle($avxpos-5,$avypos-5,$avxpos+195,$avypos+85,$white);
		$overlay->rectangle($avxpos-5,$avypos-5,$avxpos+195,$avypos+85,$black);
	        $overlay->string(gdSmallFont, $avxpos, $avypos,    "              High: $high"."ms", $black);
	        $overlay->string(gdSmallFont, $avxpos, $avypos+11, "               Low: $low"."ms", $black);
	        $overlay->string(gdSmallFont, $avxpos, $avypos+21, "           Average: $average"."ms", $black);
	        $overlay->string(gdSmallFont, $avxpos, $avypos+32, "            > $lat"."ms: $highcount pkts", $black);
	        $overlay->string(gdSmallFont, $avxpos, $avypos+42, "            < $lat"."ms: $lowcount pkts", $black);
	        $overlay->string(gdSmallFont, $avxpos, $avypos+52, " Percent over $lat" . "ms: $percentover"."%", $black);
	        $overlay->string(gdSmallFont, $avxpos, $avypos+62, "  Point references: $count pkts", $black);
	        $overlay->string(gdSmallFont, $gwidth-260, $gheight-14, "Generated on $ltime, EDT", $black);
		open(OVERLAY, ">$graphimage") or die ("Can't open graph for writing");
		binmode OVERLAY;
		print OVERLAY $overlay->png;
		close(OVERLAY);
	} else {
		open(GRAPH, ">$graphimage") or die ("Can't open graph for writing");
        	binmode GRAPH;
        	print GRAPH $image->png;
        	close(GRAPH);
	}
	return;
	
	DB_FAILURE:
		# Had a DB error.  Handle as configured.
  		print "ERROR: Database Call Failed!\n  $DBI::errstr\n";
  		return ("Failure");
 
}


getopt('gshftlm', \%opts);

if (!%opts) {
	print "$0 Usage\n\n";
	print "\t-g WxH			- Size of graph, e.g. 1024x768\n";
	print "\t-s n			- Scale height in milliseconds\n";
	print "\t-h n.n.n.n		- Hosts to include, comma separated. MUST MATCH CAPTURED DATA HOSTS. Omit for all.\n";
	print "\t-f path/file		- Output path/filename\n";
	print "\t-l <string>		- Graph Title\n";
	print "\t-m n			- Median latency (Default: 50ms)\n";
	print "\t-t n(m|h|d|w|mo)	- Timeframe in minutes,hours,days,weeks,months from now.\n\n";
	print "Example: $0 -g800x600 -s600 -h10.18.1.1,10.20.1.1 -f /var/www/html/graph.png -l \"Test Graph\" -t5d\n\n";
	exit;
}

foreach my $opt ( keys %opts ) {
	if ($opt eq 'g') {
		($gwidth, $gheight) = split('x', $opts{$opt});
	}elsif ($opt eq 's') {
		$ymax = $opts{$opt};
	}elsif ($opt eq 'h') {
		@manhosts = split(',', $opts{$opt});
	}elsif ($opt eq 'f') {
		$graphimage = $opts{$opt};
	}elsif ($opt eq 'l') {
		$title = $opts{$opt};
	}elsif ($opt eq 'm') {
		$lat = $opts{$opt};
	}elsif ($opt eq 't') {
		($opts{$opt} =~ /(\d+)(\w+)/);
		$tframe = ($1 * 60) if $2 eq 'm';
		$tframe = ($1 * 3600) if $2 eq 'h';
		$tframe = ($1 * 86400) if $2 eq 'd';
		$tframe = ($1 * 604800) if $2 eq 'w';
		$tframe = ($1 * 2678400) if $2 eq 'mo';
	}
}

#print "$gwidth, $gheight, $ymax. @manhosts, $graphimage, $tframe\n";
		
our $dbh = db_connect(0) or goto DB_FAILURE;
&get_hosts;
&make_graphs;
db_disconnect;
